bash-validator
==============

**bash-validator** is an overlay of shellcheck which is a tool for static analysis of shell code in order
to avoid certain bugs and help you follow good practices.

This project will efficiently take care of finding all shell scripts in a given tree and running shellcheck
with the correct parameters.

Installation for repository
----------------------------

Clone the repository (require to clone submodules) or download a version, then run:

```bash
sudo bash bash-validator-install.sh
```

To remove the applications, use:

```bash
sudo bash /usr/local/share/bash-validator/bash-validator-remove.sh
```

Install script will install `shellcheck` (package managers `apt-get`, `pkg` or `yum` are supported)

More detail about `shellcheck` installation: <https://github.com/koalaman/shellcheck#user-content-installing>

### Installation options

You can use following command to get help:

```bash
bash ./bash-validator-install.sh --help
```

| Option                               | Description |
|--------------------------------------|-------------|
| -h, --help                           | Display help message and exit. |
| -S, --skip-update                    | Skip package manager update |
| -P PMEXE, --package-manager PMEXE    | Force package manager binary name |
| -G, --install-shellcheck-from-github | Install shellcheck from github (othewise use package manager) |


Usage
-----

Just launch `bash-validator` from any folder, `bash-validator` will check
any shell script it can find.

```bash
# cd your_source_root_folder

bash-validator
```

If you want to restrict analysis to `*.sh` files only.

```bash
bash-validator --sh-only [-v|-vv|--verbose]
```


Options
-------

`bash-validator` options:

| Option                        | Description                   |
|-------------------------------|-------------------------------|
| `--help`                      | Display help message and exit |
| `--sh-only`                   | Limit analysis to files with `.sh` extension. |
| `-v` or `--verbose`           | Verbose output |
| `-vv`                         | More verbose   |
| `-d DIR` or `--workspace DIR` | Allow to specify root directory for analysis. If not set use current path. |
| `--ignore-default-config`     | Ignore default configuration   |
| `--ignore-user-config`        | Ignore USER configuration      |
| `--ignore-workspace-config`   | Ignore WORKSPACE configuration |


Configuration
-------------

If file `.bash-validator-ignore` exists in current directory (could be overwrited by `--workspace`),
content is take in account instead of global configuration.

You can use `bash-validator-add-to-ignore` to manage this file

```bash
bash-validator-add-to-ignore --workspace PATH_OF_WORKSPACE '*/.build/*' '*/target/*' './relative-path/to/file-to-ignore'
```

If you need to ignore some files using absolute path, you can set configuration at user level:

```bash
bash-validator-add-to-ignore --user '*/.build/*' '*/target/*' '/absolute-path/to/file-to-ignore' './relative-path/to/file-to-ignore'
```

`bash-validator-add-to-ignore` options:

| Option                        | Description                    |
|-------------------------------|--------------------------------|
| `--help`                      | Display help message and exit. |
| `--user`                      | Apply configuration at USER level. |
| `--workspace` DIR             | Define workspace directory (if ommited use current directory). |
| `--normalize`                 | Normalize configuration file. |


Related documentation
---------------------

* [Let's do Bash right!](https://github.com/progrium/bashstyle): bashstyle documentation,
* [Beginner Mistakes](http://wiki.bash-hackers.org/scripting/newbie_traps),
* [The Bash Hackers Wiki](http://wiki.bash-hackers.org/).
