#!/bin/bash -e
#
# bash-validator:bash-validator-install.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090 disable=SC1091
source "${PROJECT_HOME}/bin/install-config.sh" || exit $?
# shellcheck disable=SC1090 disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1090 disable=SC1091
source "${PROJECT_HOME}/apis/install-api/install-api.sh" || exit $?
# shellcheck disable=SC1090 disable=SC1091
source "${PROJECT_HOME}/bin/api/install-shellcheck-api.sh" || exit $?

function usage {
  cat <<EOF
Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) [--skip-update] [--package-manager PMEXE] [--install-shellcheck-from-github|--install-shellcheck-from-package-manager]

Options:
  -h,       --help  Display help message and exit.
  -G,       --install-shellcheck-from-github
                    Install shellcheck from github (default)
  -M,       --install-shellcheck-from-package-manager
                    Install shellcheck from package manager (othewise use github)
  -S,       --skip-update
                    Skip package manager update
  -P PMEXE, --package-manager PMEXE
                    Force package manager binary name

EOF
}

function main {
  local param=
  local param_pkg_manager=
  local param_shellcheck_from_github='true'
  local param_skip_update=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help') usage ; exit 0 ;;
      '-G'|'--install-shellcheck-from-github') param_shellcheck_from_github='true' ;;
      '-M'|'--install-shellcheck-from-package-manager') param_shellcheck_from_github='false' ;;
      '-P'|'--package-manager') param_pkg_manager="$( P "$@" )" && shift ;;
      '-S'|'--skip-update') param_skip_update='true' ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  exit_if_not_root
  remove_previous_version_before_install

  if [ "${param_skip_update}" != 'true' ] ; then
    install_update "${param_pkg_manager}"
  fi

  if [ "${param_shellcheck_from_github}" = 'true' ] ; then
    echoinfo 'Install lastest version from github'

    install_install_if_needed "${param_pkg_manager}" ''
    install_shellcheck_from_github "${param_pkg_manager}"
  else
    echowarn 'Install version from package manager (--install-shellcheck-from-github missing)'
    install_install_if_needed "${param_pkg_manager}" "${INSTALL_PACKAGES[@]}"
  fi

  install_main || return $?
}

main "$@" || exit $?
